#include <arduinoFFT.h>

#define SAMPLES 128             // SAMPLES-pt FFT. Must be a base 2 number. Max 128 for Arduino Uno.
#define SAMPLING_FREQUENCY 2048 // Ts = Based on Nyquist, must be 2 times the highest expected frequency.

arduinoFFT FFT = arduinoFFT();

unsigned int samplingPeriod;
unsigned long microSeconds;
int redled = 11;
int Eled = 3;
int Aled = 4;
int Dled = 5;
int Gled = 6;
int Bled = 7;
int buttonPin = 2;  // Pin connected to the push button
int count = 0;      // Counter variable
bool buttonState = false;  // Current state of the button
bool lastButtonState = false;  // Previous state of the button

double vReal[SAMPLES]; // Create vector of size SAMPLES to hold real values
double vImag[SAMPLES]; // Create vector of size SAMPLES to hold imaginary values

void setup()
{
  Serial.begin(115200); // Baud rate for the Serial Monitor
  samplingPeriod = round(1000000 * (1.0 / SAMPLING_FREQUENCY)); // Period in microseconds
  pinMode(buttonPin, INPUT_PULLUP); // Initialize the button pin as an input with a pull-up resistor
  pinMode(redled, OUTPUT); // Initialize the red LED pin as an output
}

void loop()
{
  /*Sample SAMPLES times*/
  for (int i = 0; i < SAMPLES; i++)
  {
    microSeconds = micros();    // Returns the number of microseconds since the Arduino board began running the current script.

    vReal[i] = analogRead(0); // Reads the value from analog pin 0 (A0), quantize it and save it as a real term.
    vImag[i] = 0; // Makes imaginary term 0 always

    /*remaining wait time between samples if necessary*/
    while (micros() < (microSeconds + samplingPeriod))
    {
      // Do nothing
    }
  }

  /* Perform FFT on samples */
  FFT.Windowing(vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.Compute(vReal, vImag, SAMPLES, FFT_FORWARD);
  FFT.ComplexToMagnitude(vReal, vImag, SAMPLES);

  /* Find peak frequency and print peak */
  double peak = FFT.MajorPeak(vReal, SAMPLES, SAMPLING_FREQUENCY);
  if (peak < 365)
    Serial.println(peak);

  // Control the LED based on the count value
  if (count == 0)//Button count value dictates which string is being tuned (number of LEDs tells which string)
  {
    digitalWrite(Eled, HIGH);
    if(peak>162&&peak<170)//tune low E string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }
   else if (count == 1)
  {
    digitalWrite(Aled, HIGH);
    if(peak>221&&peak<229)//tune A string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }
  else if (count == 2)
  {
    digitalWrite(Dled, HIGH);
    if(peak>296&&peak<304)//tune D string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }
 else if (count == 3)
  {
    digitalWrite(Gled, HIGH);
    if(peak>195&&peak<204)//tune G string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }
 else if (count == 4)
  {
    digitalWrite(Bled, HIGH);
    if(peak>245&&peak<250)//tune B string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }
  else if (count == 5)
  {
    digitalWrite(Eled, LOW);
    digitalWrite(Aled, LOW);
    digitalWrite(Dled, LOW);
    digitalWrite(Gled, LOW);
    digitalWrite(Bled, LOW);
    if(peak>328&&peak<337)//tune E string
    {
    digitalWrite(redled, HIGH);
    delay(2000);
    digitalWrite(redled, LOW);
    }
  }

  

  // Check if the button is pressed
  buttonState = digitalRead(buttonPin);
  if (buttonState != lastButtonState && buttonState == LOW)
  {
    // Button is pressed, increase count by 1
    count++;
    // Check if count exceeds the maximum value (5)
    if (count > 5)
    {
      count = 0; // Reset count to 0
    }
  }
  // Update last button state
  lastButtonState = buttonState;

  delay(350);
}
